@@ -1,6 +1,6 @@
 // cos-dist.h -- Cosine distribution
 //
-//  Copyright (C) 2006, 2007  Miles Bader <miles@gnu.org>
+//  Copyright (C) 2006, 2007, 2010  Miles Bader <miles@gnu.org>
 //
 // This source code is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -24,22 +24,29 @@ class CosDist : Dist
 public:
 
   // Return a sample distributed around the Z-axis according to this
-  // distribution, from the uniformly distributed parameters U and V.
+  // distribution, from the uniformly distributed parameters in PARAM.
   //
-  Vec sample (float u, float v) const
+  Vec sample (const UV &param) const
   {
-    return z_normal_symm_vec (sqrt (u), v);
+    return z_normal_symm_vec (sqrt (param.u), param.v);
   }
 
   // Return a sample distributed around the Z-axis according to this
-  // distribution, from the uniformly distributed parameters U and V.
-  // Also return the PDF of the resulting sample.
+  // distribution, from the uniformly distributed parameters in PARAM.
+  // Also return the PDF of the resulting sample in _PDF.
   //
-  Vec sample (float u, float v, float &_pdf) const
+  Vec sample (const UV &param, float &_pdf) const
   {
-    float cos_theta = sqrt (u);
+    float cos_theta = sqrt (param.u);
     _pdf = pdf (cos_theta);
-    return z_normal_symm_vec (cos_theta, v);
+    return z_normal_symm_vec (cos_theta, param.v);
+  }
+
+  // Returns the PDF of a sample in direction DIR.
+  //
+  float pdf (const Vec &dir) const
+  {
+    return pdf (dir.z);
   }
 
   // Returns the pdf of a sample, if COS_THETA is the cosine between it and
