@@ -137,38 +137,27 @@ Mesh::add_normal (vert_index_t vert_index, const Vec &normal)
 // Add a triangle to the mesh
 
 void
-Mesh::add_triangle (vert_index_t v0i, vert_index_t v1i, vert_index_t v2i,
-		    const Ref<const Material> &_mat)
+Mesh::add_triangle (vert_index_t v0i, vert_index_t v1i, vert_index_t v2i)
 {
-  Ref<const Material> mat = _mat;
-
-  if (! mat)
-    mat = material;
-
-  if (! mat)
-    throw std::runtime_error ("Mesh triangle with no material");
-
-  Triangle triang (*this, v0i, v1i, v2i, mat);
-
+  Triangle triang (*this, v0i, v1i, v2i);
   triangles.push_back (triang);
 }
 
 void
 Mesh::add_triangle (const Pos &v0, const Pos &v1, const Pos &v2,
-		    VertexGroup &vgroup, const Ref<const Material> &mat)
+		    VertexGroup &vgroup)
 {
   vert_index_t v0i = add_vertex (v0, vgroup);
   vert_index_t v1i = add_vertex (v1, vgroup);
   vert_index_t v2i = add_vertex (v2, vgroup);
 
-  add_triangle (v0i, v1i, v2i, mat);
+  add_triangle (v0i, v1i, v2i);
 }
 
 void
-Mesh::add_triangle (const Pos &v0, const Pos &v1, const Pos &v2,
-		    const Ref<const Material> &mat)
+Mesh::add_triangle (const Pos &v0, const Pos &v1, const Pos &v2)
 {
-  add_triangle (add_vertex (v0), add_vertex (v1), add_vertex (v2), mat);
+  add_triangle (add_vertex (v0), add_vertex (v1), add_vertex (v2));
 }
 
 
@@ -437,10 +426,10 @@ Mesh::Triangle::IsecInfo::make_intersect (const Media &media, RenderContext &con
 
   // Make the intersect object.
   //
-  Intersect isec (ray, media, context, triangle,
+  Intersect isec (ray, media, context, *triangle->mesh.material,
 		  normal_frame, geom_frame, T, dTds,dTdt);
 
-  isec.no_self_shadowing = true;
+  isec.no_self_shadowing = triangle;
   isec.smoothing_group = static_cast<const void *>(&triangle->mesh);
 
   return isec;
@@ -470,8 +459,12 @@ Mesh::Triangle::intersects (const ShadowRay &ray, RenderContext &) const
 	  //
 	  if (real_back != ray.isec.back)
 	    {
+	      // XXX to get the other triangle, we take advantage of the
+	      // fact that Mesh::Triangle::IsecInfo::make_intersect sets
+	      // Intersect::no_self_shadowing to the triangle.
+	      //
 	      const Triangle *other_tri
-		= static_cast<const Triangle *>(ray.isec.surface);
+		= static_cast<const Triangle *>(ray.isec.no_self_shadowing);
 	      bool other_back
 		= dot (other_tri->raw_normal_unscaled(), ray.dir) > 0;
 
@@ -521,23 +514,6 @@ Mesh::Triangle::bbox () const
   return bbox;
 }
 
-// The "smoothing group" this surface belongs to, or zero if it belongs
-// to none.  The smoothing group affects shadow-casting: if two objects
-// are in the same smoothing group, they will not be shadowed by
-// back-surface shadows from each other; typically all triangles in a
-// mesh are in the same smoothing group.
-//
-const void *
-Mesh::Triangle::smoothing_group () const
-{
-  // We only need a smoothing group if the mesh uses interpolated normals.
-
-  if (! mesh.vertex_normals.empty ())
-    return static_cast<const void *>(&mesh);
-  else
-    return 0;
-}
-
 
 
 // Object for calculating vertex normals.
