@@ -11,6 +11,7 @@
 //
 
 #include "intersect.h"
+#include "sample-sphere.h"
 
 #include "point-light.h"
 
@@ -31,13 +32,22 @@ PointLight::sample (const Intersect &isec, const UV &) const
   if (isec.cos_n (lvec) > 0 && isec.cos_geom_n (lvec) > 0)
     {
       dist_t dist = lvec.length ();
-      Vec s_dir = lvec / dist;
-      return Sample (dist * dist, 1, s_dir, dist);
+      dist_t inv_dist = 1 / dist;
+      Vec s_dir = lvec * inv_dist;
+      return Sample (color * inv_dist * inv_dist, 1, s_dir, dist);
     }
 
   return Sample ();
 }
 
+// Return a "free sample" of this light.
+//
+Light::FreeSample
+PointLight::sample (const UV &, const UV &dir_param) const
+{
+  return FreeSample (color, 4*PIf, pos, sample_sphere (dir_param));
+}
+
 // Evaluate this light in direction DIR from the viewpoint of ISEC (using
 // a surface-normal coordinate system, where the surface normal is
 // (0,0,1)).
