@@ -19,13 +19,17 @@ using namespace snogray;
 
 RenderContext::RenderContext (const GlobalRenderState &_global_state)
   : scene (_global_state.scene),
-    samples (_global_state.num_samples, *_global_state.sample_gen),
+    samples (_global_state.num_samples, *_global_state.sample_gen, random),
     global_state (_global_state),
     params (_global_state.params),
     surface_integ (
-      _global_state.surface_integ_global_state->make_integrator (*this)),
+      _global_state.surface_integ_global_state
+      ? _global_state.surface_integ_global_state->make_integrator (*this)
+      : 0),
     volume_integ (
-      _global_state.volume_integ_global_state->make_integrator (*this))
+      _global_state.volume_integ_global_state
+      ? _global_state.volume_integ_global_state->make_integrator (*this)
+      : 0)
 { }
 
 RenderContext::~RenderContext ()
