@@ -33,14 +33,18 @@
 #include "scene.h"
 #include "camera.h"
 #include "light.h"
-#include "render.h"
+#include "render-mgr.h"
 #include "recover.h"
 #include "image-output.h"
 #include "image-input.h"
 #include "image-cmdline.h"
 #include "render-cmdline.h"
 #include "scene-def.h"
+#include "camera-cmds.h"
 #include "render-stats.h"
+#include "octree.h"
+#include "pos-io.h"
+#include "vec-io.h"
 
 
 using namespace snogray;
@@ -278,9 +282,31 @@ n
 s "  -s, --size=WIDTHxHEIGHT    Set image size to WIDTH x HEIGHT pixels/lines"
 s "  -w, --width=WIDTH          Set image width to WIDTH pixels"
 s "  -h, --height=HEIGHT        Set image height to HEIGHT lines"
+#if USE_THREADS
+s "  -j, --threads=NUM          Use NUM threads for rendering"
+#endif
 n
 s RENDER_OPTIONS_HELP
 n
+s "  -c, --camera=COMMANDS      Move/point the camera according to COMMANDS:"
+s "                               g X,Y,Z     Goto absolute location X, Y, Z"
+s "                               t X,Y,Z     Point at target X, Y, Z"
+s "                               m[rludfb] D Move distance D in the given dir"
+s "                                           (right, left, up, down, fwd, back)"
+s "                               m[xyz] D    Move distance D on the given axis"
+s "                               r[rlud] A   Rotate A deg in the given dir"
+s "                               ra A        Rotate A deg around center axis"
+s "                               r[xyz] A    Rotate A degrees around [xyz]-axis"
+s "                               o[xyz] A    Orbit A degrees around [xyz]-axis"
+s "                               z SCALE     Zoom by SCALE"
+s "                               l FOC_LEN   Set lens focal-length to FOCLEN"
+s "                               f F_STOP    Set lens aperture to F_STOP"
+s "                               d DIST      Set focus distance to DIST"
+s "                               a X,Y       Auto-focus at point X,Y on image"
+s "                               u SIZE      Set scene unit to SIZE, in mm"
+s "                               h           Set camera orientation to horizontal"
+s "                               v           Set camera orientation to vertic"
+n
 s SCENE_DEF_OPTIONS_HELP
 n
 s IMAGE_OUTPUT_OPTIONS_HELP
@@ -314,6 +340,10 @@ int main (int argc, char *const *argv)
     { "progress",	no_argument,	   0, 'p' },
     { "no-progress",	no_argument,	   0, 'P' },
     { "continue",	no_argument,	   0, 'C' },
+    { "camera",		required_argument, 0, 'c' },
+#if USE_THREADS
+    { "threads",	required_argument, 0, 'j' },
+#endif
 
     RENDER_LONG_OPTIONS,
     IMAGE_OUTPUT_LONG_OPTIONS,
@@ -323,7 +353,10 @@ int main (int argc, char *const *argv)
   };
   //
   char short_options[] =
-    "s:w:h:l:qpP"
+    "s:w:h:l:qpPCc:"
+#if USE_THREADS
+    "j:"
+#endif
     SCENE_DEF_SHORT_OPTIONS
     RENDER_SHORT_OPTIONS
     IMAGE_OUTPUT_SHORT_OPTIONS
@@ -337,11 +370,13 @@ int main (int argc, char *const *argv)
   unsigned width = 0, height = 0, size = DEFAULT_IMAGE_SIZE;
   LimitSpec limit_x_spec ("min-x", 0), limit_y_spec ("min-y", 0);
   LimitSpec limit_max_x_spec ("max-x", 1.0), limit_max_y_spec ("max-y", 1.0);
+  unsigned num_threads = 1;
   bool recover = false;
   Progress::Verbosity verbosity = Progress::CHATTY;
   bool progress_set = false;
   ValTable image_params, render_params;
   SceneDef scene_def;
+  std::string camera_cmds;	// User commands for the camera
 
 
   // This speeds up I/O on cin/cout by not syncing with C stdio.
@@ -373,6 +408,16 @@ int main (int argc, char *const *argv)
 			     limit_max_x_spec, limit_max_y_spec);
 	break;
 
+#if USE_THREADS
+      case 'j':
+	num_threads = clp.unsigned_opt_arg ();
+	break;
+#endif
+
+      case 'c':
+	camera_cmds += clp.opt_arg ();
+	break;
+
       case 'C':
 	recover = true;
 	break;
@@ -437,10 +482,11 @@ int main (int argc, char *const *argv)
 
   Rusage scene_beg_ru;		// start timing scene definition
 
+  // Scene object.
+  //
   Scene scene;
   Camera camera;
 
-
   // If the user specified both a width and a height, set the camera aspect
   // ratio to maintain pixels with a 1:1 aspect ratio.  We do this before
   // loading the scene so that scene camera manipulation commands can see
@@ -449,11 +495,22 @@ int main (int argc, char *const *argv)
   if (width && height)
     camera.set_aspect_ratio (float (width) / float (height));
 
-
   // Read in the scene/camera definitions
   //
   CMDLINEPARSER_CATCH (clp, scene_def.load (scene, camera));
 
+  // Do post-load scene setup (nothign can be added to scene after this).
+  //
+  Octree::BuilderFactory octree_builder_factory;
+  scene.setup (octree_builder_factory);
+
+  // Do camera manipulation specified on the command-line.
+  //
+  if (camera_cmds.length () > 0)
+    interpret_camera_cmds (camera_cmds, camera, scene);
+
+  Rusage scene_end_ru;		// stop timing scene definition
+
 
   // Enable floating-point exceptions if possible, which can help debugging.
   // Note that we do this _after_ reading the scene which helps avoid
@@ -463,7 +520,6 @@ int main (int argc, char *const *argv)
   //
   enable_fp_exceptions ();
 
-
   // If the user specified both a width and a height, set the camera aspect
   // ratio _again_ to maintain pixels with a 1:1 aspect ratio, just in case
   // the scene file tried to muck with it.
@@ -492,8 +548,6 @@ int main (int argc, char *const *argv)
 	width = unsigned (height * ar);
     }
 
-  Rusage scene_end_ru;		// stop timing scene definition
-
 
   // Set our drawing limits based on the scene size
   //
@@ -542,7 +596,7 @@ int main (int argc, char *const *argv)
   // If the scene has a non-default background alpha set, then make sure
   // there's an alpha-channel in the output image.
   //
-  if (scene.background_alpha() != 1)
+  if (render_params.get_float ("background-alpha", 1) != 1)
     image_params.set ("alpha-channel", true);
 
   // Create output image.  The size of what we output is the same as the
@@ -566,36 +620,83 @@ int main (int argc, char *const *argv)
 
   if (recover_input)
     {
-      unsigned num_recovered = recover_image (recover_input, output);
+      unsigned num_rows_recovered = recover_image (recover_input, output);
       recover_input = 0;
 
       if (! quiet)
 	cout << "* recover: " << file_name
-	     << ": Recovered " << num_recovered << " rows" << endl;
+	     << ": Recovered " << num_rows_recovered << " rows" << endl;
 
-      if (num_recovered == limit_height)
+      if (num_rows_recovered == limit_height)
 	{
 	  cout << file_name << ": Entire image was recovered, not rendering"
 	       << endl;
 	  return 0; // We want destructors to run, so must not use exit!
 	}
+
+      // Remove the recovered rows from what we will render.
+      //
+      limit_y += num_rows_recovered;
+      limit_height -= num_rows_recovered;
     }
 
   if (! quiet)
-    cout << "* scene: "
-	 << commify_with_units (scene.num_surfaces(),
-				"top-level surface", "top-level surfaces")
-	 << ", " << commify_with_units (scene.num_lights (), "light", "lights")
-	 << endl;
+    {
+      cout << "* scene: "
+	   << commify_with_units (scene.num_surfaces(),
+				  "top-level surface", "top-level surfaces")
+	   << ", "
+	   << commify_with_units (scene.num_lights (), "light", "lights")
+	   << endl;
+      cout << "* camera: at " << camera.pos
+	   << ", pointing at "
+	   << (camera.pos + camera.forward * camera.target_dist)
+	   << " (up = " << camera.up << ")"
+	   << endl;
+      if (camera.aperture)
+	cout << "* camera: aperture " << camera.aperture
+	     << ", focal distance " << camera.focus
+	     << endl;
+    }
+
+
+  // Setup rendering state.  This can take a _very_ long time, because
+  // some rendering methods do length pre-rendering computation, so time
+  // it.
+  //
+  Rusage setup_beg_ru;
+  GlobalRenderState global_render_state (scene, render_params);
+  Rusage setup_end_ru;
 
 
   RenderStats render_stats;
 
-  // Create the image.
+  // Start rendering...
   //
   Rusage render_beg_ru;
-  render (scene, camera, width, height, output, limit_x, limit_y,
-	  render_params, render_stats, cout, verbosity);
+
+  // The pattern of pixels we will render; we add a small margin around
+  // the output image to keep the edges clean.
+  //
+  unsigned margin = output.filter_radius ();
+  RenderPattern pattern (limit_x - margin, limit_y - margin,
+			 limit_width + margin*2, limit_height + margin*2);
+
+  // Start progress indicator
+  //
+  Progress prog (std::cout, "rendering...",
+		 pattern.position (pattern.begin ()),
+		 (pattern.position (pattern.end ())
+		  - pattern.position (pattern.begin ())),
+		 verbosity);
+
+  // Do the actual rendering.
+  //
+  RenderMgr render_mgr (global_render_state, camera, width, height);
+  render_mgr.render (num_threads, pattern, output, prog, render_stats);
+
+  // Done rendering.
+  //
   Rusage render_end_ru;
 
 
@@ -612,12 +713,24 @@ int main (int argc, char *const *argv)
 
       // a field width of 14 is enough for over a year of time...
       cout << "Time:" << endl;
+
+      // Note because scene-loading often involves significant disk I/O, we
+      // add system time as well (this usually isn't a factor for other
+      // time periods we measure).
+      //
       Timeval scene_def_time
 	= ((scene_end_ru.utime() - scene_beg_ru.utime())
 	   + (scene_end_ru.stime() - scene_beg_ru.stime()));
-      cout << "  scene def cpu:" << setw (14) << scene_def_time << endl;
+      if (scene_def_time > 1)
+	cout << "  scene def cpu:" << setw (14) << scene_def_time << endl;
+
+      Timeval setup_time = setup_end_ru.utime() - setup_beg_ru.utime();
+      if (setup_time > 1)
+	cout << "  setup cpu:    " << setw (14) << setup_time << endl;
+
       Timeval render_time = render_end_ru.utime() - render_beg_ru.utime();
       cout << "  rendering cpu:" << setw (14) << render_time << endl;
+
       Timeval elapsed_time = end_time - beg_time;
       cout << "  total elapsed:" << setw (14) << elapsed_time << endl;
 
