@@ -1,6 +1,6 @@
 // surface-group.h -- Group of surfaces
 //
-//  Copyright (C) 2007, 2008  Miles Bader <miles@gnu.org>
+//  Copyright (C) 2007, 2008, 2010  Miles Bader <miles@gnu.org>
 //
 // This source code is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -28,7 +28,7 @@ class SurfaceGroup : public Surface
 {
 public:
 
-  SurfaceGroup () : Surface (0) { }
+  SurfaceGroup () { }
   ~SurfaceGroup ();
 
   // Add SURFACE to this group.
